{% load django_listing %}
{% load i18n %}
{% block group_by %}
    <div class="row">
        <div class="col-12">
            <h3>Group By</h3>
            <select class="group-by-select" multiple
                    available-title="{% trans "[Available columns]" %}"
                    selected-title="{% trans "[Selected columns]" %}"
            >
                {% for name, label, selected in listing.gb_cols_dlb %}
                    <option value="{{ name }}"{% if selected %} selected{% endif %}>{{ label }}</option>
                {% endfor %}
            </select>
        </div>
        <div class="col-12">
            <h3>Annotations</h3>
            <select class="annotation-select" multiple
                    available-title="{% trans "[Available columns]" %}"
                    selected-title="{% trans "[Selected columns]" %}"
            >
                {% for name, label, selected in listing.gb_annotate_cols_dlb %}
                    <option value="{{ name }}"{% if selected %} selected{% endif %}>{{ label }}</option>
                {% endfor %}
            </select>
        </div>
    </div>
    <div class="row">
        <div class="col text-center">
            <a class="btn btn-primary apply-group-by">Apply Grouping</a>
            <a class="btn btn-primary remove-group-by">Remove Grouping</a>
        </div>
    </div>
{% endblock group_by %}